@@ -7,11 +7,12 @@ use anyhow::Result;
 use fuel_asm::Opcode;
 use fuel_gql_client::client::FuelClient;
 use fuel_tx::{
-    AssetId, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId, Witness,
+    Address, AssetId, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId, Witness,
 };
-use fuel_types::{Bytes32, Immediate12, Salt, Word};
-use fuel_vm::consts::{REG_CGAS, REG_RET, REG_ZERO, VM_TX_MEMORY};
+use fuel_types::{Bytes32, Salt, Word};
+use fuel_vm::consts::{REG_CGAS, REG_ONE};
 use fuel_vm::prelude::Contract as FuelContract;
+use fuel_vm::script_with_data_offset;
 use fuels_core::{
     constants::DEFAULT_COIN_AMOUNT, constants::WORD_SIZE, Detokenize, Selector, Token,
 };
@@ -74,48 +75,33 @@ impl Contract {
         fuel_client: &FuelClient,
         tx_parameters: TxParameters,
         call_parameters: CallParameters,
+        variable_outputs: Option<Vec<Output>>,
         maturity: Word,
         compute_calldata_offset: bool,
         external_contracts: Option<Vec<ContractId>>,
         wallet: LocalWallet,
     ) -> Result<Vec<Receipt>, Error> {
-        // The script len is defined by the 6 Opcodes in the `script`, which is `24`
-        // plus `asset_id_offset`, which is `32`, totalling 56.
-        let script_len = 56;
-        let script_data_offset = VM_TX_MEMORY + Transaction::script_offset() + script_len;
-        let script_data_offset = script_data_offset as Immediate12;
-
-        // The offset that locates the asset ID in the script data.
-        // It goes from the beginning of `script_data` to `32`.
-        let asset_id_offset = 32;
-
         // Script to call the contract. The offset that points to the `script_data` is loaded at the
         // register `0x12`. Note that we're picking `0x12` simply because it could be any
         // non-reserved register. Then, we use the Opcode to call a contract: `CALL` pointing at the
         // register that we loaded the `script_data` at.
-        #[allow(clippy::iter_cloned_collect)]
-        let script = vec![
-            // Setting `0x10` to point to the 32-byte asset ID of the amount to forward.
-            Opcode::ADDI(0x10, REG_ZERO, asset_id_offset),
-            // Setting `0x11` to hold the amount of of coins to forward.
-            Opcode::ADDI(0x11, REG_ZERO, call_parameters.amount as Immediate12),
-            // Setting `0x12` to the `script_data`, defined down below but
-            // we already know its length.
-            Opcode::ADDI(0x12, REG_ZERO, script_data_offset),
-            Opcode::CALL(0x12, 0x11, 0x10, REG_CGAS),
-            Opcode::RET(REG_RET),
-            Opcode::NOOP,
-        ]
-        .iter()
-        .copied()
-        .collect::<Vec<u8>>();
-
-        assert_eq!(
-            script.len(),
-            script_len - asset_id_offset as usize,
-            "Script length *must* be 24"
+        let (script, offset) = script_with_data_offset!(
+            data_offset,
+            vec![
+                // Load call data to 0x10.
+                Opcode::MOVI(0x10, data_offset + 32),
+                // Load gas forward to 0x11.
+                Opcode::MOVI(0x12, call_parameters.amount as Immediate18),
+                // Load the asset id to use to 0x13.
+                Opcode::MOVI(0x13, data_offset),
+                // Call the transfer contract.
+                Opcode::CALL(0x10, 0x12, 0x13, REG_CGAS),
+                Opcode::RET(REG_ONE),
+            ]
         );
 
+        let script = script.iter().copied().collect::<Vec<u8>>();
+
         // `script_data` consists of:
         // 1. Asset ID to be forwarded
         // 2. Contract ID (ContractID::LEN);
@@ -142,7 +128,7 @@ impl Contract {
         // transaction. If it doesn't take any custom inputs, this isn't necessary.
         if compute_calldata_offset {
             // Offset of the script data relative to the call data
-            let call_data_offset = script_data_offset as usize + ContractId::LEN + 2 * WORD_SIZE;
+            let call_data_offset = (offset + 32) as usize + ContractId::LEN + 2 * WORD_SIZE;
             let call_data_offset = call_data_offset as Word;
 
             script_data.extend(&call_data_offset.to_be_bytes());
@@ -215,6 +201,11 @@ impl Contract {
             }
         }
 
+        // Add outputs to the transaction.
+        if let Some(v) = variable_outputs {
+            outputs.extend(v);
+        };
+
         let mut tx = Transaction::script(
             tx_parameters.gas_price,
             tx_parameters.gas_limit,
@@ -276,6 +267,7 @@ impl Contract {
             fuel_client: provider.client.clone(),
             datatype: PhantomData,
             output_params: output_params.to_vec(),
+            variable_outputs: None,
             compute_calldata_offset,
             external_contracts: None,
             wallet: wallet.clone(),
@@ -376,6 +368,7 @@ pub struct ContractCall<D> {
     pub output_params: Vec<ParamType>,
     pub compute_calldata_offset: bool,
     pub wallet: LocalWallet,
+    pub variable_outputs: Option<Vec<Output>>,
     external_contracts: Option<Vec<ContractId>>,
 }
 
@@ -411,6 +404,26 @@ where
         self
     }
 
+    /// Appends `num` `Output::Variable`s to the transaction.
+    /// Note that this is a builder method, i.e. use it as a chain:
+    /// `my_contract_instance.my_method(...).add_variable_outputs(num).call()`.
+    pub fn append_variable_outputs(mut self, num: u64) -> Self {
+        let new_outputs: Vec<Output> = (0..num)
+            .map(|_| Output::Variable {
+                amount: 0,
+                to: Address::zeroed(),
+                asset_id: AssetId::default(),
+            })
+            .collect();
+
+        match self.variable_outputs {
+            Some(ref mut outputs) => outputs.extend(new_outputs),
+            None => self.variable_outputs = Some(new_outputs),
+        }
+
+        self
+    }
+
     /// Call a contract's method. Return a Result<CallResponse, Error>.
     /// The CallResponse structs contains the method's value in its `value`
     /// field as an actual typed value `D` (if your method returns `bool`, it will
@@ -425,6 +438,7 @@ where
             &self.fuel_client,
             self.tx_parameters,
             self.call_parameters,
+            self.variable_outputs,
             self.maturity,
             self.compute_calldata_offset,
             self.external_contracts,
